;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Instance <Map_S_T> of the theory FiniteMap
;; Map_S_T-template.smt

 :extrasorts (<Map_S_T>)

;; The map domain

 :extrafuns ((domain_<Map_S_T> <Map_S_T> <Set_S>))

;; Map application

 :extrafuns ((apply_<Map_S_T> <Map_S_T> <S> <T>))

;; Empty maps

 :extrafuns ((emptyMap_<Map_S_T> <Map_S_T>))

 :assumption (= (domain_<Map_S_T> emptyMap_<Map_S_T>) emptySet_<Set_S>)

;; Map overwriting

 :extrafuns ((overwrite_<Map_S_T> <Map_S_T> <S> <T> <Map_S_T>))

 :assumption (forall (?x <Map_S_T>) (?y <S>) (?z <T>)
                     (= (apply_<Map_S_T> (overwrite_<Map_S_T> ?x ?y ?z) ?y) ?z)
              :pat { (overwrite_<Map_S_T> ?x ?y ?z) } )

 :assumption (forall (?x <Map_S_T>) (?y1 <S>) (?y2 <S>) (?z <T>)
                     (or (= ?y1 ?y2)
                         (not (in_<Set_S> ?y2 (domain_<Map_S_T> ?x)))
                         (= (apply_<Map_S_T> (overwrite_<Map_S_T> ?x ?y1 ?z) ?y2)
                            (apply_<Map_S_T> ?x ?y2)))
              :pat { (apply_<Map_S_T> (overwrite_<Map_S_T> ?x ?y1 ?z) ?y2) } )

 :assumption (forall (?x <Map_S_T>) (?y <S>) (?z <T>)
                     (= (domain_<Map_S_T> (overwrite_<Map_S_T> ?x ?y ?z))
                        (insert_<Set_S> ?y (domain_<Map_S_T> ?x)))
              :pat { (domain_<Map_S_T> (overwrite_<Map_S_T> ?x ?y ?z)) } )

;; Extensionality

 :extrapreds ((equals_<Map_S_T> <Map_S_T> <Map_S_T>))
 :assumption (forall (?x <Map_S_T>) (?y <Map_S_T>)
               (iff (equals_<Map_S_T> ?x ?y)
                    (and (= (domain_<Map_S_T> ?x) (domain_<Map_S_T> ?y))
                         (forall (?z <S>)
                                 (implies (in_<Set_S> ?z (domain_<Map_S_T> ?x))
                                          (= (apply_<Map_S_T> ?x ?z)
                                             (apply_<Map_S_T> ?y ?z))))))
              :pat { (equals_<Map_S_T> ?x ?y) } )

 :assumption (forall (?x <Map_S_T>) (?y <Map_S_T>)
                     (iff (= ?x ?y) (equals_<Map_S_T> ?x ?y)))

;; The map range

 :extrafuns ((range_<Map_S_T> <Map_S_T> <Set_T>))

 :assumption (= (range_<Map_S_T> emptyMap_<Map_S_T>) emptySet_<Set_T>)

 :assumption (forall (?x <Map_S_T>) (?y <S>) (?z <T>)
                     (= (range_<Map_S_T> (overwrite_<Map_S_T> ?x ?y ?z))
                        (insert_<Set_T> ?z (range_<Map_S_T> ?x)))
              :pat { (range_<Map_S_T> (overwrite_<Map_S_T> ?x ?y ?z)) } )

;; Restriction of the domain of maps

 :extrafuns ((restrict_<Map_S_T> <Map_S_T> <Set_S> <Map_S_T>))

 :assumption (forall (?x <Map_S_T>) (?y <Set_S>)
                     (= (domain_<Map_S_T> (restrict_<Map_S_T> ?x ?y))
                        (inter_<Set_S> (domain_<Map_S_T> ?x) ?y))
              :pat { (restrict_<Map_S_T> ?x ?y) } )

 :assumption (forall (?x <Map_S_T>) (?y <Set_S>) (?z <S>)
                     (implies (and (in_<Set_S> ?z (domain_<Map_S_T> ?x))
                                   (in_<Set_S> ?z ?y))
                              (= (apply_<Map_S_T> (restrict_<Map_S_T> ?x ?y) ?z)
                                 (apply_<Map_S_T> ?x ?z)))
              :pat { (apply_<Map_S_T> (restrict_<Map_S_T> ?x ?y) ?z) } )

;; Subtraction of the domain of maps

 :extrafuns ((subtract_<Map_S_T> <Map_S_T> <Set_S> <Map_S_T>))

 :assumption (forall (?x <Map_S_T>) (?y <Set_S>)
                     (= (domain_<Map_S_T> (subtract_<Map_S_T> ?x ?y))
                        (setminus_<Set_S> (domain_<Map_S_T> ?x) ?y))
              :pat { (subtract_<Map_S_T> ?x ?y) } )

 :assumption (forall (?x <Map_S_T>) (?y <Set_S>) (?z <S>)
                     (implies (and (in_<Set_S> ?z (domain_<Map_S_T> ?x))
                                   (not (in_<Set_S> ?z ?y)))
                              (= (apply_<Map_S_T> (subtract_<Map_S_T> ?x ?y) ?z)
                                 (apply_<Map_S_T> ?x ?z)))
              :pat { (apply_<Map_S_T> (subtract_<Map_S_T> ?x ?y) ?z) } )

;; End: Instance <Map_S_T> of the theory FiniteMap
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
